@@ -2,15 +2,18 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Experimental SPF support.
+/* SPF support.
+   Copyright (c) The Exim Maintainers 2016 - 2024
    Copyright (c) Tom Kistner <tom@duncanthrax.net> 2004
-   License: GPL */
+   License: GPL
+   SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
-#ifdef EXPERIMENTAL_SPF
+#ifdef SUPPORT_SPF
 
 /* Yes, we do have ns_type. spf.h redefines it if we don't set this. Doh */
-#ifndef HAVE_NS_TYPE
-#define HAVE_NS_TYPE
+#if !defined(HAVE_NS_TYPE) && defined(NS_INADDRSZ)
+# define HAVE_NS_TYPE
 #endif
 #include <spf2/spf.h>
 
@@ -23,8 +26,11 @@ typedef struct spf_result_id {
 } spf_result_id;
 
 /* prototypes */
-int spf_init(uschar *,uschar *);
-int spf_process(uschar **, uschar *, int);
+gstring * spf_lib_version_report(gstring *);
+BOOL spf_init(void);
+BOOL spf_conn_init(uschar *, uschar *);
+int  spf_process(const uschar **, const uschar *, int);
+void spf_response_debug(SPF_response_t *);
 
 #define SPF_PROCESS_NORMAL  0
 #define SPF_PROCESS_GUESS   1
